@@ -29,6 +29,11 @@
 //#define BG_PRINT_PARAM_CLASSIC
 //#define BG_UNIT_LRA
 
+//#define BG_LOUDNESS_ALL
+#define BG_TRADITIONAL
+#if ! defined(BG_TRADITIONAL) // [
+#define BG_OPPOSITE
+#endif // ]
 #define BG_ESCAPE_XML
 #if defined (_WIN32) // [
 #define BG_COMPILE_TIME_EXPR
@@ -925,6 +930,11 @@ struct bg_param {
 #endif // ]
   double norm;
   double preamp;
+#if defined (BG_TRADITIONAL) // [
+  int traditional;
+#elif defined (BG_OPPOSITE) // ] [
+  int opposite;
+#endif // ]
 
   struct {
     int enabled;
