@@ -67,7 +67,11 @@ static void bg_root_track_id(bg_tree_t *tree FFUNUSED, int *id FFUNUSED)
 #endif // ]
 
 static bg_tree_vmt_t bg_root_vmt={
+#if defined (PBU_MESSAGE_AV_LOG) // [
+  .id="root",
+#else // ] [
   .id=FFL("root"),
+#endif // ]
   .type=BG_TREE_TYPE_ROOT,
   .destroy=bg_root_destroy,
   .accept=bg_root_accept,
