@@ -21,7 +21,19 @@
  */
 #include <bg.h>
 
+#if 1 // {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
+#undef FF_BITMASK_BASED_CHANNEL_LAYOUT
+#endif // ]
+#if defined (FF_HOLZHAMMER) // [
+#undef FF_HOLZHAMMER
+#endif // ]
+#endif // }
+
+#if defined (FF_HOLZHAMMER) // {
 FF_DISABLE_DEPRECATION_WARNINGS // [
+#endif // }
+
 ///////////////////////////////////////////////////////////////////////////////
 static bg_tree_vmt_t bg_track_vmt;
 static ff_input_callback_t bg_input_callback;
@@ -90,10 +102,17 @@ int bg_track_content_create(bg_tree_t *tree)
 #endif // ]
 
   /////////////////////////////////////////////////////////////////////////////
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   if (!track->input.audio.ctx->channel_layout) {
     _DMESSAGE("missing input channel layout");
     goto e_channel_layout;
   }
+#else // ] [
+  if (!track->input.audio.ctx->ch_layout.nb_channels) {
+    _DMESSAGE("missing input channel layout");
+    goto e_channel_layout;
+  }
+#endif // ]
 
   /////////////////////////////////////////////////////////////////////////////
   track->root.id=++param->count.cur;
@@ -244,9 +263,15 @@ static int input_stats_create(void *data, const AVCodecParameters *codecpar)
 {
   bg_tree_t *tree=data;
   bg_track_t *track=&tree->track;
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   int channels=codecpar->channels<FF_LFE_THRESHOLD
       ?codecpar->channels
       :codecpar->channels-1;
+#else // ] [
+  int channels=codecpar->ch_layout.nb_channels<FF_LFE_THRESHOLD
+      ?codecpar->ch_layout.nb_channels
+      :codecpar->ch_layout.nb_channels-1;
+#endif // ]
 
   /////////////////////////////////////////////////////////////////////////////
   if (tree->stats.momentary||tree->stats.shortterm) {
@@ -319,7 +344,7 @@ static void input_stats_destroy(void *data)
     lib1770_pre_close(track->filter.pre);
 }
 
-static int input_stats_add(void *data, int upsampled, AVFrame *frame)
+static int input_stats_add(void *data FFUNUSED, int upsampled FFUNUSED, AVFrame *frame FFUNUSED)
 {
   bg_tree_t *tree=data;
   bg_track_t *track=&tree->track;
@@ -424,4 +449,6 @@ static ff_input_callback_t bg_input_callback={
 #endif // ]
 };
 
+#if defined (FF_HOLZHAMMER) // {
 FF_ENABLE_DEPRECATION_WARNINGS // ]
+#endif // }
