@@ -21,28 +21,43 @@
  */
 #include <ff.h>
 
+#if 1 // {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
+#undef FF_BITMASK_BASED_CHANNEL_LAYOUT
+#endif // ]
+#if defined (FF_HOLZHAMMER) // [
+#undef FF_HOLZHAMMER
+#endif // ]
+#endif // }
+
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT)
 FF_DISABLE_DEPRECATION_WARNINGS // [
+#endif
 ///////////////////////////////////////////////////////////////////////////////
 #define FF_ANALYZER_CREATE_OCHANNELS
 #define FF_ANALYZER_OCODECPAR_PTR
+#define FF_ANALYZER_STATS_CREATE
+//#define FF_ANALYZER_STATS_DESTROY
 int ff_analyzer_create(ff_analyzer_t *a, ff_inout_t *in)
 {
   ff_input_callback_t *cb=in->cb.in;
   void *data=in->cb.data;
   int upsample=cb&&cb->upsample?cb->upsample(data):-1;
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   const ff_param_decode_t *(*decode)(const void *)=cb?cb->decode:NULL;
-//#if defined (FF_CHANNEL_LAYOUT_DEPRECATED) // [
-//  int64_t channel_layout=decode?
-//			ff_layout2int64(&decode(data)->request.ch_layout):0ll;
-//#else // ] [
   int64_t channel_layout=decode?decode(data)->request.channel_layout:-1ll;
-//#endif // ]
+#endif // ]
+#if defined (FF_ANALYZER_STATS_CREATE) // [ 
   int (*create)(void *, const AVCodecParameters *)=cb?cb->stats.create:NULL;
+#endif // ]
+#if defined (FF_ANALYZER_STATS_DESTROY) // [
+  void (*destroy)(void *data) FFUNUSED=cb?cb->stats.destroy:NULL;
+#endif // ]
 #if defined (FF_ANALYZER_OCODECPAR_PTR) // [
   AVCodecParameters *icodecpar,*ocodecpar;
 #else // ] [
   AVCodecParameters *icodecpar,ocodecpar;
-#endif // ]
+#endif // err
 
   /////////////////////////////////////////////////////////////////////////////
   if (in->ai<0) {
@@ -53,23 +68,30 @@ int ff_analyzer_create(ff_analyzer_t *a, ff_inout_t *in)
   /////////////////////////////////////////////////////////////////////////////
   icodecpar=in->fmt.ctx->streams[in->ai]->codecpar;
 
-#if defined (FF_CHANNEL_LAYOUT_DEPRECATED) // [
-  if (ff_layout2int64(&icodecpar->ch_layout)) {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
+  if (!icodecpar->channel_layout) {
+    _DMESSAGE("missing input channel layout");
+    goto e_args;
+  }
+#elif 0 // ] [
+  if (AV_CHANNEL_ORDER_UNSPEC==icodecpar->ch_layout.order) {
     _DMESSAGE("missing input channel layout");
     goto e_args;
   }
 #else // ] [
-  if (!icodecpar->channel_layout) {
+  if (!icodecpar->ch_layout.nb_channels) {
     _DMESSAGE("missing input channel layout");
     goto e_args;
   }
 #endif // ]
 
   /////////////////////////////////////////////////////////////////////////////
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   if (!channel_layout) {
     _DMESSAGE("missing output channel layout");
     goto e_args;
   }
+#endif // ]
 
   /////////////////////////////////////////////////////////////////////////////
   if (!in->audio.ctx) {
@@ -82,7 +104,9 @@ int ff_analyzer_create(ff_analyzer_t *a, ff_inout_t *in)
   a->in=in;
 
   /////////////////////////////////////////////////////////////////////////////
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   if (0ll<channel_layout&&channel_layout!=(int)icodecpar->channel_layout) {
+#endif // ]
 #if defined (FF_ANALYZER_OCODECPAR_PTR) // [
     /*
      * from "avcodec.h":
@@ -104,9 +128,11 @@ int ff_analyzer_create(ff_analyzer_t *a, ff_inout_t *in)
       goto e_normalizer_ocodecpar_copy;
     }
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
     ocodecpar->channel_layout=channel_layout;
 #if defined (FF_ANALYZER_CREATE_OCHANNELS) // [
     ocodecpar->channels=av_get_channel_layout_nb_channels(channel_layout);
+#endif // ]
 #endif // ]
 
     ///////////////////////////////////////////////////////////////////////////
@@ -116,12 +142,14 @@ int ff_analyzer_create(ff_analyzer_t *a, ff_inout_t *in)
       goto e_normalizer;
     }
 
+#if defined (FF_ANALYZER_STATS_CREATE) // [ 
     ///////////////////////////////////////////////////////////////////////////
     if (create&&create(data,ocodecpar)<0) {
       _DMESSAGE("creating statistics");
       avcodec_parameters_free(&ocodecpar);
       goto e_stats;
     }
+#endif // ]
 
     avcodec_parameters_free(&ocodecpar);
 #else // ] [
@@ -143,17 +171,23 @@ int ff_analyzer_create(ff_analyzer_t *a, ff_inout_t *in)
       goto e_stats;
     }
 #endif // ]
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   }
   else {
+#endif // ]
     ///////////////////////////////////////////////////////////////////////////
     a->normalizer.ctx=NULL;
 
+#if 0 && defined (FF_ANALYZER_STATS_CREATE) // [ 
     ///////////////////////////////////////////////////////////////////////////
     if (create&&create(data,icodecpar)<0) {
       _DMESSAGE("creating statistics");
       goto e_stats;
     }
+#endif // ]
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   }
+#endif // ]
 
   /////////////////////////////////////////////////////////////////////////////
   a->pkt=av_packet_alloc();
@@ -203,7 +237,6 @@ int ff_analyzer_create(ff_analyzer_t *a, ff_inout_t *in)
       goto e_upsampler;
     }
 
-    avcodec_parameters_free(&ocodecpar);
 #else // ] [
     ocodecpar=*icodecpar;
     ocodecpar.sample_rate*=upsample;
@@ -422,8 +455,16 @@ static int ff_analyzer_send_packet(ff_analyzer_t *a, AVPacket *pkt)
         a->state=FF_ANALYZER_DECODER_SEND_PACKET;
         return err;
       default:
+#if defined (FF_MUXEER_CONTINUE_ON_INVALIDDATA) // [
+	// try again.
+      	_DWARNINGV("receiving frame: %s (%d:\"%s\")",av_err2str(err),err,
+              a->in->cb.in->path(a->in->cb.data));
+    	a->state=FF_ANALYZER_DECODER_SEND_PACKET;
+	continue;
+#else // ] [
         _DMESSAGEV("receiving frame: %s (%d)",av_err2str(err),err);
         return err;
+#endif // ] 
       }
     default:
       _DMESSAGE("unexpected state");
@@ -524,4 +565,6 @@ e_loop:
   return err;
 }
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT)
 FF_ENABLE_DEPRECATION_WARNINGS // ]
+#endif
