@@ -21,7 +21,19 @@
  */
 #include <ff.h>
 
+#if 0 // {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
+#undef FF_BITMASK_BASED_CHANNEL_LAYOUT
+#endif // ]
+#if defined (FF_HOLZHAMMER) // [
+#undef FF_HOLZHAMMER
+#endif // ]
+#endif // }
+
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // {
 FF_DISABLE_DEPRECATION_WARNINGS // [
+#endif // }
+
 ///////////////////////////////////////////////////////////////////////////////
 // https://0xdeafc0de.wordpress.com/2014/07/21/seeking-with-ffmpeg/
 //#define FF_SEEK_DEFAULT
@@ -186,6 +198,7 @@ int ff_input_create(ff_inout_t *in, ff_input_callback_t *cb, void *data,
         continue;
       }
 #else // ] [
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
       if (istream->codecpar->channels<=0) {
         _DWARNINGV("channel underflow: %d",istream->codecpar->channels);
         continue;
@@ -194,23 +207,22 @@ int ff_input_create(ff_inout_t *in, ff_input_callback_t *cb, void *data,
         _DWARNINGV("channel overflow: %d",istream->codecpar->channels);
         continue;
       }
+#else // ] [
+      if (istream->codecpar->ch_layout.nb_channels<=0) {
+        _DWARNINGV("channel underflow: %d",istream->codecpar->ch_layout.nb_channels);
+        continue;
+      }
+      else if (AV_NUM_DATA_POINTERS<istream->codecpar->ch_layout.nb_channels) {
+        _DWARNINGV("channel overflow: %d",istream->codecpar->ch_layout.nb_channels);
+        continue;
+      }
+#endif // ]
 #endif // ]
 #if defined (FF_SERGEY_INDEX_BUGFIX) // [
 			else if (i<in->ai)
 				continue;
 #endif // ]
-#if defined (FF_CHANNEL_LAYOUT_DEPRECATED) // [
-      else if (in->ai<0&&istream->codecpar->ch_layout.nb_channels) {
-        // if no audio stream found yet, pick this one (regardless whether
-        // stereo or not).
-        in->ai=i;
-      }
-      else if (2==istream->codecpar->ch_layout.nb_channels) {
-        // if an audio stream alreay was found (and is non-stereo),
-        // replace it by this stereo one.
-        in->ai=i;
-      }
-#else // ] [
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
       else if (in->ai<0&&istream->codecpar->channels) {
         // if no audio stream found yet, pick this one (regardless whether
         // stereo or not).
@@ -221,6 +233,17 @@ int ff_input_create(ff_inout_t *in, ff_input_callback_t *cb, void *data,
         // replace it by this stereo one.
         in->ai=i;
       }
+#else // ] [
+      else if (in->ai<0&&istream->codecpar->ch_layout.nb_channels) {
+        // if no audio stream found yet, pick this one (regardless whether
+        // stereo or not).
+        in->ai=i;
+      }
+      else if (2==istream->codecpar->ch_layout.nb_channels) {
+        // if an audio stream alreay was found (and is non-stereo),
+        // replace it by this stereo one.
+        in->ai=i;
+      }
 #endif // ]
 
       break;
@@ -405,9 +428,13 @@ fprintf(stderr,"(%s,%d,%s) hack=%d\n",pbu_basename(__FILE__),__LINE__,__func__,h
 fprintf(stderr,"(%s,%d,%s) hack=%d\n",pbu_basename(__FILE__),__LINE__,__func__,hack);fflush(stderr);
 #endif // ]
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   if (hack&&d)
     d->request.sample_fmt=codecpar->format;
-
+#else // ] [
+  if (hack&&d)
+    d->request.ch_layout=codecpar->ch_layout;
+#endif // ]
   return hack;
 }
 
@@ -709,7 +736,11 @@ static int ff_output_add_astream(ff_inout_t *out)
   ff_input_callback_t *icb=in->cb.in;
   void *idata=in->cb.data;
   const ff_param_decode_t *decode=icb&&icb->decode?icb->decode(idata):NULL;
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   int64_t channel_layout=decode?decode->request.channel_layout:-1ll;
+#else // ] [
+  AVChannelLayout ch_layout=decode->request.ch_layout;
+#endif // ]
   AVStream *istream=in->fmt.ctx->streams[in->ai];
   AVStream *ostream;
 #if defined (FF_CODECPAR_ALLOC) // [
@@ -788,11 +819,12 @@ static int ff_output_add_astream(ff_inout_t *out)
     codecpar->format=sample_fmt;
 #endif // ]
 
-#if defined (FF_CHANNEL_LAYOUT_DEPRECATED) // [
-	#error not implemented yet.
-#else // ] [
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
     if (0ll<=channel_layout)
       codecpar->channel_layout=channel_layout;
+#else // ] [
+    if (av_channel_layout_check(&ch_layout))
+      codecpar->ch_layout=ch_layout;
 #endif // ]
 
     err=ff_audio_create(&out->audio,out,NULL,codecpar);
@@ -1008,4 +1040,6 @@ void ff_output_destroy(ff_inout_t *out)
   avformat_free_context(out->fmt.ctx);
 }
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // {
 FF_ENABLE_DEPRECATION_WARNINGS // ]
+#endif // }
