@@ -22,7 +22,19 @@
 #include <lib1770.h>
 #include <ff.h>
 
+#if 1 // {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
+#undef FF_BITMASK_BASED_CHANNEL_LAYOUT
+#endif // ]
+#if defined (FF_HOLZHAMMER) // [
+#undef FF_HOLZHAMMER
+#endif // ]
+#endif // }
+
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // {
 FF_DISABLE_DEPRECATION_WARNINGS // [
+#endif // }
+
 ///////////////////////////////////////////////////////////////////////////////
 static ff_iter_vmt_t ff_iter_s16i_vmt;
 static ff_iter_vmt_t ff_iter_s32i_vmt;
@@ -150,7 +162,11 @@ static void ff_iter_s16i_first(ff_iter_t *i)
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic pop
 #endif // ]
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.s16.mp=i->i.s16.rp+i->frame->channels*i->frame->nb_samples;
+#else // ] [
+  i->i.s16.mp=i->i.s16.rp+i->frame->ch_layout.nb_channels*i->frame->nb_samples;
+#endif // ]
 }
 
 static int ff_iter_s16i_valid(ff_iter_t *i)
@@ -160,7 +176,11 @@ static int ff_iter_s16i_valid(ff_iter_t *i)
 
 static void ff_iter_s16i_next(ff_iter_t *i)
 {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.s16.rp+=i->frame->channels;
+#else // ] [
+  i->i.s16.rp+=i->frame->ch_layout.nb_channels;
+#endif // ]
 }
 
 static void ff_iter_s16i_norm(ff_iter_t *i, double *x, double *max)
@@ -168,6 +188,7 @@ static void ff_iter_s16i_norm(ff_iter_t *i, double *x, double *max)
   int ch;
   int16_t sample;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
     sample=i->i.s16.rp[ch];
 
@@ -176,6 +197,18 @@ static void ff_iter_s16i_norm(ff_iter_t *i, double *x, double *max)
     else
       ff_iter_norm(i,x,max,i->frame->channels,ch,FF_INT16_SCALE*sample);
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+    sample=i->i.s16.rp[ch];
+
+    if (sample<0)
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT16_SCALE_NEG*sample);
+    else
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT16_SCALE*sample);
+  }
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_s16i_vmt={
@@ -197,7 +230,11 @@ static void ff_iter_s32i_first(ff_iter_t *i)
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic pop
 #endif // ]
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.s32.mp=i->i.s32.rp+i->frame->channels*i->frame->nb_samples;
+#else // ] [
+  i->i.s32.mp=i->i.s32.rp+i->frame->ch_layout.nb_channels*i->frame->nb_samples;
+#endif // ]
 }
 
 static int ff_iter_s32i_valid(ff_iter_t *i)
@@ -207,7 +244,11 @@ static int ff_iter_s32i_valid(ff_iter_t *i)
 
 static void ff_iter_s32i_next(ff_iter_t *i)
 {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.s32.rp+=i->frame->channels;
+#else // ] [
+  i->i.s32.rp+=i->frame->ch_layout.nb_channels;
+#endif // ]
 }
 
 static void ff_iter_s32i_norm(ff_iter_t *i, double *x, double *max)
@@ -215,6 +256,7 @@ static void ff_iter_s32i_norm(ff_iter_t *i, double *x, double *max)
   int ch;
   int32_t sample;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
     sample=i->i.s32.rp[ch];
 
@@ -223,6 +265,18 @@ static void ff_iter_s32i_norm(ff_iter_t *i, double *x, double *max)
     else
       ff_iter_norm(i,x,max,i->frame->channels,ch,FF_INT32_SCALE*sample);
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+    sample=i->i.s32.rp[ch];
+
+    if (sample<0)
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT32_SCALE_NEG*sample);
+    else
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT32_SCALE*sample);
+  }
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_s32i_vmt={
@@ -244,7 +298,11 @@ static void ff_iter_flti_first(ff_iter_t *i)
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic pop
 #endif // ]
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.flt.mp=i->i.flt.rp+i->frame->channels*i->frame->nb_samples;
+#else // ] [
+  i->i.flt.mp=i->i.flt.rp+i->frame->ch_layout.nb_channels*i->frame->nb_samples;
+#endif // ]
 }
 
 static int ff_iter_flti_valid(ff_iter_t *i)
@@ -254,15 +312,24 @@ static int ff_iter_flti_valid(ff_iter_t *i)
 
 static void ff_iter_flti_next(ff_iter_t *i)
 {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.flt.rp+=i->frame->channels;
+#else // ] [
+  i->i.flt.rp+=i->frame->ch_layout.nb_channels;
+#endif // ]
 }
 
 static void ff_iter_flti_norm(ff_iter_t *i, double *x, double *max)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ff_iter_norm(i,x,max,i->frame->channels,ch,i->i.flt.rp[ch]);
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,i->i.flt.rp[ch]);
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_flti_vmt={
@@ -284,7 +351,11 @@ static void ff_iter_dbli_first(ff_iter_t *i)
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic pop
 #endif // ]
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.dbl.mp=i->i.dbl.rp+i->frame->channels*i->frame->nb_samples;
+#else // ] [
+  i->i.dbl.mp=i->i.dbl.rp+i->frame->ch_layout.nb_channels*i->frame->nb_samples;
+#endif // ]
 }
 
 static int ff_iter_dbli_valid(ff_iter_t *i)
@@ -294,15 +365,24 @@ static int ff_iter_dbli_valid(ff_iter_t *i)
 
 static void ff_iter_dbli_next(ff_iter_t *i)
 {
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   i->i.dbl.rp+=i->frame->channels;
+#else // ] [
+  i->i.dbl.rp+=i->frame->ch_layout.nb_channels;
+#endif // ]
 }
 
 static void ff_iter_dbli_norm(ff_iter_t *i, double *x, double *max)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ff_iter_norm(i,x,max,i->frame->channels,ch,i->i.dbl.rp[ch]);
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,i->i.dbl.rp[ch]);
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_dbli_vmt={
@@ -318,6 +398,7 @@ static void ff_iter_s16p_first(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic push
@@ -328,6 +409,18 @@ static void ff_iter_s16p_first(ff_iter_t *i)
 #pragma GCC diagnostic pop
 #endif // ]
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif // ]
+    i->p.s16.rp[ch]=(int16_t *)i->frame->data[ch];
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic pop
+#endif // ]
+  }
+#endif // ]
 
   i->p.s16.mp=i->p.s16.rp[0]+i->frame->nb_samples;
 }
@@ -341,8 +434,13 @@ static void ff_iter_s16p_next(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ++i->p.s16.rp[ch];
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ++i->p.s16.rp[ch];
+#endif // ]
 }
 
 static void ff_iter_s16p_norm(ff_iter_t *i, double *x, double *max)
@@ -350,6 +448,7 @@ static void ff_iter_s16p_norm(ff_iter_t *i, double *x, double *max)
   int ch;
   int16_t sample;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
     sample=*i->p.s16.rp[ch];
 
@@ -358,6 +457,18 @@ static void ff_iter_s16p_norm(ff_iter_t *i, double *x, double *max)
     else
       ff_iter_norm(i,x,max,i->frame->channels,ch,FF_INT16_SCALE*sample);
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+    sample=*i->p.s16.rp[ch];
+
+    if (sample<0)
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT16_SCALE_NEG*sample);
+    else
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT16_SCALE*sample);
+  }
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_s16p_vmt={
@@ -373,6 +484,7 @@ static void ff_iter_s32p_first(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic push
@@ -383,6 +495,18 @@ static void ff_iter_s32p_first(ff_iter_t *i)
 #pragma GCC diagnostic pop
 #endif // ]
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif // ]
+    i->p.s32.rp[ch]=(int32_t *)i->frame->data[ch];
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic pop
+#endif // ]
+  }
+#endif // ]
 
   i->p.s32.mp=i->p.s32.rp[0]+i->frame->nb_samples;
 }
@@ -396,8 +520,13 @@ static void ff_iter_s32p_next(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ++i->p.s32.rp[ch];
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ++i->p.s32.rp[ch];
+#endif // ]
 }
 
 static void ff_iter_s32p_norm(ff_iter_t *i, double *x, double *max)
@@ -405,6 +534,7 @@ static void ff_iter_s32p_norm(ff_iter_t *i, double *x, double *max)
   int ch;
   int32_t sample;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
     sample=*i->p.s32.rp[ch];
 
@@ -413,6 +543,18 @@ static void ff_iter_s32p_norm(ff_iter_t *i, double *x, double *max)
     else
       ff_iter_norm(i,x,max,i->frame->channels,ch,FF_INT32_SCALE*sample);
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+    sample=*i->p.s32.rp[ch];
+
+    if (sample<0)
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT32_SCALE_NEG*sample);
+    else
+      ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,
+	  FF_INT32_SCALE*sample);
+  }
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_s32p_vmt={
@@ -428,6 +570,7 @@ static void ff_iter_fltp_first(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic push
@@ -438,6 +581,18 @@ static void ff_iter_fltp_first(ff_iter_t *i)
 #pragma GCC diagnostic pop
 #endif // ]
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif // ]
+    i->p.flt.rp[ch]=(float *)i->frame->data[ch];
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic pop
+#endif // ]
+  }
+#endif // ]
 
   i->p.flt.mp=i->p.flt.rp[0]+i->frame->nb_samples;
 }
@@ -451,16 +606,26 @@ static void ff_iter_fltp_next(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ++i->p.flt.rp[ch];
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ++i->p.flt.rp[ch];
+#endif // ]
 }
 
 static void ff_iter_fltp_norm(ff_iter_t *i, double *x, double *max)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ff_iter_norm(i,x,max,i->frame->channels,ch,*i->p.flt.rp[ch]);
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,*i->p.flt.rp[ch]);
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_fltp_vmt={
@@ -476,6 +641,7 @@ static void ff_iter_dblp_first(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch) {
 #if defined (__GNUC__) // [
 #pragma GCC diagnostic push
@@ -486,6 +652,18 @@ static void ff_iter_dblp_first(ff_iter_t *i)
 #pragma GCC diagnostic pop
 #endif // ]
   }
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch) {
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif // ]
+    i->p.dbl.rp[ch]=(double *)i->frame->data[ch];
+#if defined (__GNUC__) // [
+#pragma GCC diagnostic pop
+#endif // ]
+  }
+#endif // ]
 
   i->p.dbl.mp=i->p.dbl.rp[0]+i->frame->nb_samples;
 }
@@ -499,16 +677,26 @@ static void ff_iter_dblp_next(ff_iter_t *i)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ++i->p.dbl.rp[ch];
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ++i->p.dbl.rp[ch];
+#endif // ]
 }
 
 static void ff_iter_dblp_norm(ff_iter_t *i, double *x, double *max)
 {
   int ch;
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // [
   for (ch=0;ch<i->frame->channels;++ch)
     ff_iter_norm(i,x,max,i->frame->channels,ch,*i->p.dbl.rp[ch]);
+#else // ] [
+  for (ch=0;ch<i->frame->ch_layout.nb_channels;++ch)
+    ff_iter_norm(i,x,max,i->frame->ch_layout.nb_channels,ch,*i->p.dbl.rp[ch]);
+#endif // ]
 }
 
 static ff_iter_vmt_t ff_iter_dblp_vmt={
@@ -551,4 +739,6 @@ static ff_iter_vmt_t ff_iter_err_vmt={
   .norm=ff_iter_err_norm,
 };
 
+#if defined (FF_BITMASK_BASED_CHANNEL_LAYOUT) // {
 FF_ENABLE_DEPRECATION_WARNINGS // ]
+#endif // }
