@@ -235,7 +235,16 @@ static int ff_muxer_send_packet(ff_muxer_t *m, AVPacket *pkt)
       case AVERROR_INVALIDDATA:
         // Invalid data found when processing input.
         // No respective documentation found. May occur at the end of MP3.
+#if defined (FF_MUXEER_CONTINUE_ON_INVALIDDATA) // [
+	// try again.
+      	_DWARNINGV("sending packet: %s (%d:\"%s\")",av_err2str(err),err,
+              m->in->cb.in->path(m->in->cb.data));
+        m->state=FF_MUXER_DECODER_SEND_PACKET;
+        continue;
+#else // ] [
         // Intentional fall-through.
+        _DMESSAGEV("sending packet: %s (%d)",av_err2str(err),err);
+#endif // ]
       case 0:
         // 0 on success
         m->state=FF_MUXER_DECODER_RECEIVE_FRAME;
