@@ -34,6 +34,7 @@ extern "C" {
 
 ///////////////////////////////////////////////////////////////////////////////
 //#define PBU_DEBUG
+#define PBU_MESSAGE_AV_LOG
 #define PBU_LIST_RING
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -50,6 +51,23 @@ extern "C" {
 #define PBU_FREE_GUARDED(p) \
   ((p)?free(p),(p)=NULL:(p))
 
+///////////////////////////////////////////////////////////////////////////////
+#define __DMESSAGE(format) (\
+	av_log(NULL, AV_LOG_ERROR,"Error: " format " (%s:%d:%s)\n",\
+			pbu_basename(__FILE__),__LINE__,__func__)\
+)
+#define __DMESSAGEV(format,...) (\
+	av_log(NULL, AV_LOG_ERROR,"Error: " format " (%s:%d:%s)\n",\
+			__VA_ARGS__,pbu_basename(__FILE__),__LINE__,__func__)\
+)
+#define __DWARNING(format) (\
+	av_log(NULL, AV_LOG_WARNING,"Warning: " format " (%s:%d:%s)\n",\
+			pbu_basename(__FILE__),__LINE__,__func__)\
+)
+#define __DWARNINGV(format,...) (\
+	av_log(NULL, AV_LOG_WARNING,"Warning: " format " (%s:%d:%s)\n",\
+			__VA_ARGS__,pbu_basename(__FILE__),__LINE__,__func__)\
+)
 ///////////////////////////////////////////////////////////////////////////////
 #if defined (_WIN32) // [
 #define PBU_PRIs "S"
@@ -62,6 +80,8 @@ extern "C" {
   fwprintf(f,L##format,__VA_ARGS__),\
   fflush(f)\
 )
+//#if defined (PBU_MESSAGE_AV_LOG) // [
+//#else // ] [
 #define _DMESSAGE(format) (\
   fwprintf(stderr,L"Error: " L##format L" (%s:%s:%s)\n",\
     pbu_basename(__FILE__),PBU_STR(__LINE__),__func__),\
@@ -82,6 +102,7 @@ extern "C" {
     __VA_ARGS__,pbu_basename(__FILE__),PBU_STR(__LINE__),__func__),\
   fflush(stderr)\
 )
+//#endif // ]
 
 #define _SNPRINTF(s,size,format,...) snwprintf(s,size,L##format,__VA_ARGS__)
 
